# -*- coding: utf-8 -*-
import os
from typing import List, Iterable
from urllib.parse import urlparse
from concurrent.futures import ThreadPoolExecutor, as_completed

import requests
from requests.models import Response

from abeja.common.config import DEFAULT_CHUNK_SIZE, FETCH_WORKER_COUNT
from abeja.common.iterator import Iterator
from abeja.common.connection import http_error_handler
from abeja.common.local_file import (
    use_binary_cache,
    use_text_cache,
    use_iter_content_cache,
    use_iter_lines_cache
)
from abeja.common.file_utils import FileMixin
from abeja.datalake.api.client import APIClient


class DatalakeFile(FileMixin):
    def __init__(self, api: APIClient, organization_id: str=None, channel_id: str=None,
                 file_id: str=None, uri: str=None, type: str=None,
                 upload_url: str=None, download_url: str=None, content_type: str=None,
                 url_expires_on: str=None, metadata: dict=None, uploaded_at: str=None,
                 **kwargs):
        super(DatalakeFile, self).__init__(api, uri, type, **kwargs)
        self.organization_id = organization_id
        if self.uri:
            pr = urlparse(self.uri)
            self.channel_id = pr.netloc
            self.file_id = self._convert_to_file_id(pr.path)
        else:
            self.channel_id = channel_id
            self.file_id = file_id
        self.upload_url = upload_url
        self.download_url = download_url
        self.content_type = content_type
        self.url_expires_on = url_expires_on
        self.metadata = metadata
        self.uploaded_at = uploaded_at

    @use_binary_cache
    def get_content(self) -> bytes:
        """get content from a binary file
        if the binary file exists in local, get from the file.
        unless, get from remote, and save it in local.
        """
        return self._get_content_from_remote()

    @use_iter_content_cache
    def get_iter_content(self, chunk_size: int=DEFAULT_CHUNK_SIZE) -> Iterable[bytes]:
        return self._get_iter_content_from_remote(chunk_size)

    @use_text_cache
    def get_text(self) -> str:
        """get content from a text file
        if the text file exists in local, get from the file.
        unless, get from remote, and save it in local.
        """
        return self._get_text_from_remote()

    def get_json(self) -> dict:
        return self._get_json_from_remote()

    @use_iter_lines_cache
    def get_iter_lines(self) -> Iterable[str]:
        return self._get_iter_lines_from_remote()

    def get_file_info(self) -> dict:
        """get information of a file

        response of `/channels/<channel_id>/<file_id>`
        {
            "url_expires_on": "2017-12-20T17:08:26+00:00",
            "uploaded_at": "2017-12-18T05:39:47+00:00",
            "metadata": {
                "x-abeja-meta-filename": "test.jpg"
            },
            "file_id": "20171218T053947-821bd0a3-3992-4320-bc1c-1ee8d0a0ad6b",
            "download_url": "..."
            "content_type": "image/jpeg"
        }

        :return:
        """
        path = os.path.join('/channels', self.channel_id, self.file_id)
        return self._api._connection.api_request(method='GET', path=path)

    def _get_content_from_remote(self) -> bytes:
        res = self._do_download()
        return res.content

    def _get_iter_content_from_remote(self, chunk_size) -> Iterable[bytes]:
        res = self._do_download(stream=True)
        return res.iter_content(chunk_size=chunk_size)

    def _get_text_from_remote(self) -> str:
        res = self._do_download()
        return res.text

    def _get_json_from_remote(self) -> dict:
        res = self._do_download()
        return res.json()

    def _get_iter_lines_from_remote(self) -> Iterable[str]:
        res = self._do_download(stream=True)
        return res.iter_lines()

    def _convert_to_file_id(self, path: str) -> str:
        """path is like `/<file_id>`, convert it in valid format.

        :param path:
        :return:
        """
        return path.lstrip('/')

    def _get_download_url(self) -> str:
        file_info = self.get_file_info()
        return file_info['download_url']

    def _do_download(self, stream: bool=False) -> Response:
        url = self._get_download_url()
        try:
            return self._api._connection.request('GET', url, stream=stream)
        except requests.exceptions.HTTPError as e:
            http_error_handler(e)


def _download_file_content(item: DatalakeFile) -> DatalakeFile:
    # download content and cache to local disk
    item.get_content()
    return item


class FileIterator(Iterator):
    def __init__(self, api: APIClient, organization_id: str, channel_id: str,
                 start: str=None, end: str=None, timezone: str=None,
                 items_per_page: int=None, next_page_token: str=None, prefetch=False):
        self._api = api
        self.organization_id = organization_id
        self.channel_id = channel_id
        self.start = start
        self.end = end
        self.timezone = timezone
        self.limit = items_per_page
        self.next_page_token = next_page_token
        self._prev_page_token = None
        self._page_counter = 1
        self.prefetch = prefetch
        super().__init__()

    def __iter__(self):
        if self.prefetch:
            return self._items_iter_with_prefetch()
        else:
            return self._items_iter()

    def _items_iter_with_prefetch(self) -> Iterable[DatalakeFile]:
        page = self._page()
        with ThreadPoolExecutor(max_workers=FETCH_WORKER_COUNT) as executor:
            futures = []
            while page:
                futures += [
                    executor.submit(_download_file_content, item) for item in page
                ]
                page = self._page()
            for f in as_completed(futures):
                download_item = f.result()
                yield download_item

    def _page(self) -> List[DatalakeFile]:
        """get a page of items in channel"""
        params = {}
        if self.next_page_token:
            params['next_page_token'] = self.next_page_token
        if self.limit:
            params['limit'] = self.limit
        res = self._api.list_channel_files(self.channel_id, **params)

        # In the case that the number of files to be returned is below limit,
        # with next_page_token is None, the response is composed of next_page_token is None
        # and files.
        # Because of this behavior, every time this method is called, the files to be returned
        # will be those of the last time, which will lead finite loop in `_page_iter`
        # This is why return empty list under the below condition.
        self._previous_page_token = self.next_page_token
        self.next_page_token = res.get('next_page_token')
        if self._prev_page_token == self.next_page_token and self._page_counter > 1:
            return []

        self._page_counter += 1
        return [DatalakeFile(self._api, self.organization_id, self.channel_id, **_item) for _item in res['files']]


class Files:
    def __init__(self, api: APIClient, organization_id: str, channel_id: str):
        self._api = api
        self.organization_id = organization_id
        self.channel_id = channel_id

    def list(self, start: str=None, end: str=None, timezone: str=None,
             next_page_token: str=None, limit: int=None, prefetch: bool=False):
        """generate all datalake files in a channel

        Request syntax:
            .. code-block:: python

                response = files.list()

        Params:
            - **start** (str): start date of target uploaded files
            - **end** (str): end date of target uploaded files
            - **timezone** (str): timezone of specified start and end date
            - **next_page_token** (str) : next page token to get the next items. **[optional]**
            - **limit** (int): limit of items. **[optional]**
            - **prefetch** :(bool)**[optional]**

        Return type:
            FileIterator
        """
        return FileIterator(self._api, self.organization_id, self.channel_id,
                            start=start, end=end, timezone=timezone, next_page_token=next_page_token,
                            items_per_page=limit, prefetch=prefetch)
